@@ -30,6 +30,7 @@ import (
 
 	"github.com/go-kit/kit/log"
 	"github.com/go-kit/kit/log/level"
+	"github.com/pkg/errors"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/client_golang/prometheus/promhttp"
 	"github.com/prometheus/common/model"
@@ -37,6 +38,8 @@ import (
 	promlogflag "github.com/prometheus/common/promlog/flag"
 	"github.com/prometheus/common/route"
 	"github.com/prometheus/common/version"
+	"github.com/prometheus/exporter-toolkit/web"
+	webflag "github.com/prometheus/exporter-toolkit/web/kingpinflag"
 	"gopkg.in/alecthomas/kingpin.v2"
 
 	"github.com/prometheus/alertmanager/api"
@@ -46,9 +49,18 @@ import (
 	"github.com/prometheus/alertmanager/inhibit"
 	"github.com/prometheus/alertmanager/nflog"
 	"github.com/prometheus/alertmanager/notify"
+	"github.com/prometheus/alertmanager/notify/email"
+	"github.com/prometheus/alertmanager/notify/opsgenie"
+	"github.com/prometheus/alertmanager/notify/pagerduty"
+	"github.com/prometheus/alertmanager/notify/pushover"
+	"github.com/prometheus/alertmanager/notify/slack"
+	"github.com/prometheus/alertmanager/notify/victorops"
+	"github.com/prometheus/alertmanager/notify/webhook"
+	"github.com/prometheus/alertmanager/notify/wechat"
 	"github.com/prometheus/alertmanager/provider/mem"
 	"github.com/prometheus/alertmanager/silence"
 	"github.com/prometheus/alertmanager/template"
+	"github.com/prometheus/alertmanager/timeinterval"
 	"github.com/prometheus/alertmanager/types"
 	"github.com/prometheus/alertmanager/ui"
 )
@@ -70,12 +82,33 @@ var (
 		},
 		[]string{"handler", "method"},
 	)
+	clusterEnabled = prometheus.NewGauge(
+		prometheus.GaugeOpts{
+			Name: "alertmanager_cluster_enabled",
+			Help: "Indicates whether the clustering is enabled or not.",
+		},
+	)
+	configuredReceivers = prometheus.NewGauge(
+		prometheus.GaugeOpts{
+			Name: "alertmanager_receivers",
+			Help: "Number of configured receivers.",
+		},
+	)
+	configuredIntegrations = prometheus.NewGauge(
+		prometheus.GaugeOpts{
+			Name: "alertmanager_integrations",
+			Help: "Number of configured integrations.",
+		},
+	)
 	promlogConfig = promlog.Config{}
 )
 
 func init() {
 	prometheus.MustRegister(requestDuration)
 	prometheus.MustRegister(responseSize)
+	prometheus.MustRegister(clusterEnabled)
+	prometheus.MustRegister(configuredReceivers)
+	prometheus.MustRegister(configuredIntegrations)
 	prometheus.MustRegister(version.NewCollector("alertmanager"))
 }
 
@@ -92,6 +125,52 @@ func instrumentHandler(handlerName string, handler http.HandlerFunc) http.Handle
 
 const defaultClusterAddr = "0.0.0.0:9094"
 
+// buildReceiverIntegrations builds a list of integration notifiers off of a
+// receiver config.
+func buildReceiverIntegrations(nc *config.Receiver, tmpl *template.Template, logger log.Logger) ([]notify.Integration, error) {
+	var (
+		errs         types.MultiError
+		integrations []notify.Integration
+		add          = func(name string, i int, rs notify.ResolvedSender, f func(l log.Logger) (notify.Notifier, error)) {
+			n, err := f(log.With(logger, "integration", name))
+			if err != nil {
+				errs.Add(err)
+				return
+			}
+			integrations = append(integrations, notify.NewIntegration(n, rs, name, i))
+		}
+	)
+
+	for i, c := range nc.WebhookConfigs {
+		add("webhook", i, c, func(l log.Logger) (notify.Notifier, error) { return webhook.New(c, tmpl, l) })
+	}
+	for i, c := range nc.EmailConfigs {
+		add("email", i, c, func(l log.Logger) (notify.Notifier, error) { return email.New(c, tmpl, l), nil })
+	}
+	for i, c := range nc.PagerdutyConfigs {
+		add("pagerduty", i, c, func(l log.Logger) (notify.Notifier, error) { return pagerduty.New(c, tmpl, l) })
+	}
+	for i, c := range nc.OpsGenieConfigs {
+		add("opsgenie", i, c, func(l log.Logger) (notify.Notifier, error) { return opsgenie.New(c, tmpl, l) })
+	}
+	for i, c := range nc.WechatConfigs {
+		add("wechat", i, c, func(l log.Logger) (notify.Notifier, error) { return wechat.New(c, tmpl, l) })
+	}
+	for i, c := range nc.SlackConfigs {
+		add("slack", i, c, func(l log.Logger) (notify.Notifier, error) { return slack.New(c, tmpl, l) })
+	}
+	for i, c := range nc.VictorOpsConfigs {
+		add("victorops", i, c, func(l log.Logger) (notify.Notifier, error) { return victorops.New(c, tmpl, l) })
+	}
+	for i, c := range nc.PushoverConfigs {
+		add("pushover", i, c, func(l log.Logger) (notify.Notifier, error) { return pushover.New(c, tmpl, l) })
+	}
+	if errs.Len() > 0 {
+		return nil, &errs
+	}
+	return integrations, nil
+}
+
 func main() {
 	os.Exit(run())
 }
@@ -108,13 +187,14 @@ func run() int {
 		retention       = kingpin.Flag("data.retention", "How long to keep data for.").Default("120h").Duration()
 		alertGCInterval = kingpin.Flag("alerts.gc-interval", "Interval between alert GC.").Default("30m").Duration()
 
+		webConfig      = webflag.AddFlags(kingpin.CommandLine)
 		externalURL    = kingpin.Flag("web.external-url", "The URL under which Alertmanager is externally reachable (for example, if Alertmanager is served via a reverse proxy). Used for generating relative and absolute links back to Alertmanager itself. If the URL has a path portion, it will be used to prefix all HTTP endpoints served by Alertmanager. If omitted, relevant URL components will be derived automatically.").String()
 		routePrefix    = kingpin.Flag("web.route-prefix", "Prefix for the internal routes of web endpoints. Defaults to path of --web.external-url.").String()
 		listenAddress  = kingpin.Flag("web.listen-address", "Address to listen on for the web interface and API.").Default(":9093").String()
 		getConcurrency = kingpin.Flag("web.get-concurrency", "Maximum number of GET requests processed concurrently. If negative or zero, the limit is GOMAXPROC or 8, whichever is larger.").Default("0").Int()
 		httpTimeout    = kingpin.Flag("web.timeout", "Timeout for HTTP requests. If negative or zero, no timeout is set.").Default("0").Duration()
 
-		clusterBindAddr = kingpin.Flag("cluster.listen-address", "Listen address for cluster.").
+		clusterBindAddr = kingpin.Flag("cluster.listen-address", "Listen address for cluster. Set to empty string to disable HA mode.").
 				Default(defaultClusterAddr).String()
 		clusterAdvertiseAddr = kingpin.Flag("cluster.advertise-address", "Explicit address to advertise in cluster.").String()
 		peers                = kingpin.Flag("cluster.peer", "Initial peers (may be repeated).").Strings()
@@ -130,6 +210,7 @@ func run() int {
 	)
 
 	promlogflag.AddFlags(kingpin.CommandLine, &promlogConfig)
+	kingpin.CommandLine.UsageWriter(os.Stdout)
 
 	kingpin.Version(version.Print("alertmanager"))
 	kingpin.CommandLine.GetFlag("help").Short('h')
@@ -165,6 +246,7 @@ func run() int {
 			level.Error(logger).Log("msg", "unable to initialize gossip mesh", "err", err)
 			return 1
 		}
+		clusterEnabled.Set(1)
 	}
 
 	stopc := make(chan struct{})
@@ -253,11 +335,19 @@ func run() int {
 		return disp.Groups(routeFilter, alertFilter)
 	}
 
+	// An interface value that holds a nil concrete value is non-nil.
+	// Therefore we explicly pass an empty interface, to detect if the
+	// cluster is not enabled in notify.
+	var clusterPeer cluster.ClusterPeer
+	if peer != nil {
+		clusterPeer = peer
+	}
+
 	api, err := api.New(api.Options{
 		Alerts:      alerts,
 		Silences:    silences,
 		StatusFunc:  marker.Status,
-		Peer:        peer,
+		Peer:        clusterPeer,
 		Timeout:     *httpTimeout,
 		Concurrency: *getConcurrency,
 		Logger:      log.With(logger, "component", "api"),
@@ -266,15 +356,16 @@ func run() int {
 	})
 
 	if err != nil {
-		level.Error(logger).Log("err", fmt.Errorf("failed to create API: %v", err.Error()))
+		level.Error(logger).Log("err", errors.Wrap(err, "failed to create API"))
 		return 1
 	}
 
-	amURL, err := extURL(*listenAddress, *externalURL)
+	amURL, err := extURL(logger, os.Hostname, *listenAddress, *externalURL)
 	if err != nil {
-		level.Error(logger).Log("err", err)
+		level.Error(logger).Log("msg", "failed to determine external URL", "err", err)
 		return 1
 	}
+	level.Debug(logger).Log("externalURL", amURL.String())
 
 	waitFunc := func() time.Duration { return 0 }
 	if peer != nil {
@@ -289,45 +380,101 @@ func run() int {
 
 	var (
 		inhibitor *inhibit.Inhibitor
-		silencer  *silence.Silencer
 		tmpl      *template.Template
-		pipeline  notify.Stage
 	)
 
+	dispMetrics := dispatch.NewDispatcherMetrics(prometheus.DefaultRegisterer)
+	pipelineBuilder := notify.NewPipelineBuilder(prometheus.DefaultRegisterer)
+	configLogger := log.With(logger, "component", "configuration")
 	configCoordinator := config.NewCoordinator(
 		*configFile,
 		prometheus.DefaultRegisterer,
-		log.With(logger, "component", "configuration"),
+		configLogger,
 	)
 	configCoordinator.Subscribe(func(conf *config.Config) error {
 		tmpl, err = template.FromGlobs(conf.Templates...)
 		if err != nil {
-			return fmt.Errorf("failed to parse templates: %v", err.Error())
+			return errors.Wrap(err, "failed to parse templates")
 		}
 		tmpl.ExternalURL = amURL
 
+		// Build the routing tree and record which receivers are used.
+		routes := dispatch.NewRoute(conf.Route, nil)
+		activeReceivers := make(map[string]struct{})
+		routes.Walk(func(r *dispatch.Route) {
+			activeReceivers[r.RouteOpts.Receiver] = struct{}{}
+		})
+
+		// Build the map of receiver to integrations.
+		receivers := make(map[string][]notify.Integration, len(activeReceivers))
+		var integrationsNum int
+		for _, rcv := range conf.Receivers {
+			if _, found := activeReceivers[rcv.Name]; !found {
+				// No need to build a receiver if no route is using it.
+				level.Info(configLogger).Log("msg", "skipping creation of receiver not referenced by any route", "receiver", rcv.Name)
+				continue
+			}
+			integrations, err := buildReceiverIntegrations(rcv, tmpl, logger)
+			if err != nil {
+				return err
+			}
+			// rcv.Name is guaranteed to be unique across all receivers.
+			receivers[rcv.Name] = integrations
+			integrationsNum += len(integrations)
+		}
+
+		// Build the map of time interval names to mute time definitions.
+		muteTimes := make(map[string][]timeinterval.TimeInterval, len(conf.MuteTimeIntervals))
+		for _, ti := range conf.MuteTimeIntervals {
+			muteTimes[ti.Name] = ti.TimeIntervals
+		}
+
 		inhibitor.Stop()
 		disp.Stop()
 
 		inhibitor = inhibit.NewInhibitor(alerts, conf.InhibitRules, marker, logger)
-		silencer = silence.NewSilencer(silences, marker, logger)
-		pipeline = notify.BuildPipeline(
-			conf.Receivers,
-			tmpl,
+		silencer := silence.NewSilencer(silences, marker, logger)
+
+		// An interface value that holds a nil concrete value is non-nil.
+		// Therefore we explicly pass an empty interface, to detect if the
+		// cluster is not enabled in notify.
+		var pipelinePeer notify.Peer
+		if peer != nil {
+			pipelinePeer = peer
+		}
+
+		pipeline := pipelineBuilder.New(
+			receivers,
 			waitFunc,
 			inhibitor,
 			silencer,
+			muteTimes,
 			notificationLog,
-			peer,
-			logger,
+			pipelinePeer,
 		)
+		configuredReceivers.Set(float64(len(activeReceivers)))
+		configuredIntegrations.Set(float64(integrationsNum))
 
 		api.Update(conf, func(labels model.LabelSet) {
 			inhibitor.Mutes(labels)
 			silencer.Mutes(labels)
 		})
 
-		disp = dispatch.NewDispatcher(alerts, dispatch.NewRoute(conf.Route, nil), pipeline, marker, timeoutFunc, logger)
+		disp = dispatch.NewDispatcher(alerts, routes, pipeline, marker, timeoutFunc, logger, dispMetrics)
+		routes.Walk(func(r *dispatch.Route) {
+			if r.RouteOpts.RepeatInterval > *retention {
+				level.Warn(configLogger).Log(
+					"msg",
+					"repeat_interval is greater than the data retention period. It can lead to notifications being repeated more often than expected.",
+					"repeat_interval",
+					r.RouteOpts.RepeatInterval,
+					"retention",
+					*retention,
+					"route",
+					r.Key(),
+				)
+			}
+		})
 
 		go disp.Run()
 		go inhibitor.Run()
@@ -340,15 +487,17 @@ func run() int {
 	}
 
 	// Make routePrefix default to externalURL path if empty string.
-	if routePrefix == nil || *routePrefix == "" {
+	if *routePrefix == "" {
 		*routePrefix = amURL.Path
 	}
-
 	*routePrefix = "/" + strings.Trim(*routePrefix, "/")
+	level.Debug(logger).Log("routePrefix", *routePrefix)
 
 	router := route.New().WithInstrumentation(instrumentHandler)
-
 	if *routePrefix != "/" {
+		router.Get("/", func(w http.ResponseWriter, r *http.Request) {
+			http.Redirect(w, r, *routePrefix, http.StatusFound)
+		})
 		router = router.WithPrefix(*routePrefix)
 	}
 
@@ -358,12 +507,12 @@ func run() int {
 
 	mux := api.Register(router, *routePrefix)
 
-	srv := http.Server{Addr: *listenAddress, Handler: mux}
+	srv := &http.Server{Addr: *listenAddress, Handler: mux}
 	srvc := make(chan struct{})
 
 	go func() {
 		level.Info(logger).Log("msg", "Listening", "address", *listenAddress)
-		if err := srv.ListenAndServe(); err != http.ErrServerClosed {
+		if err := web.ListenAndServe(srv, *webConfig, logger); err != http.ErrServerClosed {
 			level.Error(logger).Log("msg", "Listen error", "err", err)
 			close(srvc)
 		}
@@ -417,9 +566,9 @@ func clusterWait(p *cluster.Peer, timeout time.Duration) func() time.Duration {
 	}
 }
 
-func extURL(listen, external string) (*url.URL, error) {
+func extURL(logger log.Logger, hostnamef func() (string, error), listen, external string) (*url.URL, error) {
 	if external == "" {
-		hostname, err := os.Hostname()
+		hostname, err := hostnamef()
 		if err != nil {
 			return nil, err
 		}
@@ -427,6 +576,9 @@ func extURL(listen, external string) (*url.URL, error) {
 		if err != nil {
 			return nil, err
 		}
+		if port == "" {
+			level.Warn(logger).Log("msg", "no port found for listen address", "address", listen)
+		}
 
 		external = fmt.Sprintf("http://%s:%s/", hostname, port)
 	}
@@ -435,6 +587,9 @@ func extURL(listen, external string) (*url.URL, error) {
 	if err != nil {
 		return nil, err
 	}
+	if u.Scheme != "http" && u.Scheme != "https" {
+		return nil, errors.Errorf("%q: invalid %q scheme, only 'http' and 'https' are supported", u.String(), u.Scheme)
+	}
 
 	ppref := strings.TrimRight(u.Path, "/")
 	if ppref != "" && !strings.HasPrefix(ppref, "/") {
